# -*- coding: utf-8 -*-

# Almost all code comes from http://code.activestate.com/recipes/552751/

import sys


if sys.version_info[:2] == (2, 5) or sys.version_info[:2] == (2, 4):
    from server25 import SimpleJSONRPCServer
elif sys.version_info[:2] == (2, 6):
    from server26 import SimpleJSONRPCServer







# -*- coding: utf-8 -*-

from berrymq import (following,
                     following_function,
                     auto_twitter,
                     twitter_exception,
                     cond,
                     Follower,
                     set_multiplicity,
                     twitter,
                     regist_method,
                     regist_function)


from connect import (init_connection,
                     interconnect,
                     connect_oneway,
                     connect_via_queue,
                     close_connection,
                     send_message,
                     get, get_nowait,
                     get_matched_followers)



# -*- coding: utf-8 -*-

import json
import select 
import threading
import SocketServer
from server_common import (SimpleJSONRPCDispatcher,
                           SimpleJSONRPCRequestHandler)


class SimpleJSONRPCServer(SocketServer.TCPServer,
                          SimpleJSONRPCDispatcher):
    """Simple JSON-RPC server.

    Simple JSON-RPC server that allows functions and a single instance
    to be installed to handle requests. The default implementation
    attempts to dispatch JSON-RPC calls to the functions or instance
    installed in the server. Override the _dispatch method inhereted
    from SimpleJSONRPCDispatcher to change this behavior.
    """

    allow_reuse_address = True

    def __init__(self, addr, requestHandler=SimpleJSONRPCRequestHandler,
                 logRequests=True):
        self.logRequests = logRequests

        SimpleJSONRPCDispatcher.__init__(self, allow_none=True, encoding=None)
        SocketServer.TCPServer.__init__(self, addr, requestHandler)

        self.__thread = None  

    def serve_forever(self, in_thread=False, poll_interval=0.5):
        def serve_thread(server, poll_interval):
            server.serve_forever(poll_interval=poll_interval)
        if in_thread:
            args = [self, poll_interval]
            self.__thread = threading.Thread(target=serve_thread, args=args)
            self.__thread.setDaemon(True)
            self.__thread.start()
        else:
            SocketServer.TCPServer.serve_forever(self, poll_interval)

    def shutdown(self, immediately = True):
        if not immediately:
            self._BaseServer__serving = False
            return
        SocketServer.TCPServer.shutdown(self)
        if self.__thread:
            self.__thread.join()
            self.__thread = None



# -*- coding: utf-8 -*-


import sys
import traceback
import SocketServer
import SimpleXMLRPCServer
try:
    import json
except ImportError:
    import simplejson as json


class SimpleJSONRPCDispatcher(SimpleXMLRPCServer.SimpleXMLRPCDispatcher):
    def _marshaled_dispatch(self, data, dispatch_method = None):
        id = None
        try:
            req = json.loads(data)
            method = req['method']
            params = req['params']
            id     = req['id']

            if dispatch_method is not None:
                result = dispatch_method(method, params)
            else:
                result = self._dispatch(method, params)
            response = dict(id=id, result=result, error=None)
        except:
            extpe, exv, extrc = sys.exc_info()
            err = dict(type=str(extpe),
                       message=str(exv),
                       traceback=''.join(traceback.format_tb(extrc)))
            response = dict(id=id, result=None, error=err)
        try:
            return json.dumps(response)
        except:
            extpe, exv, extrc = sys.exc_info()
            err = dict(type=str(extpe),
                       message=str(exv),
                       traceback=''.join(traceback.format_tb(extrc)))
            response = dict(id=id, result=None, error=err)
            return json.dumps(response)


class SimpleJSONRPCRequestHandler(SimpleXMLRPCServer.SimpleXMLRPCRequestHandler):
    # Class attribute listing the accessible path components;
    # paths not on this list will result in a 404 error.
    rpc_paths = ('/', '/JSON')



import wx
import wx.lib.newevent
import berrymq

(BerryMQEvent, EVT_BERRYMQ_MSG) = wx.lib.newevent.NewEvent()


class wxPythonAdapter(object):
    def __init__(self, window, id_filter):
        self.window = window
        berrymq.regist_method(id_filter, self.listener)
        print "wxPythonAdapter, %s" % id_filter
    
    def listener(self, message):
    	print message.id
        event = BerryMQEvent(id=message.id, 
                             args=message.args, kwargs=message.kwargs)
        wx.PostEvent(self.window, event)



# -*- coding: utf-8 -*-

import sys
import simplejson as json
import select 
import threading
import SocketServer
from server_common import (SimpleJSONRPCDispatcher,
                           SimpleJSONRPCRequestHandler)


class SimpleJSONRPCServer(SocketServer.TCPServer,
                          SimpleJSONRPCDispatcher):
    """Simple JSON-RPC server.

    Simple JSON-RPC server that allows functions and a single instance
    to be installed to handle requests. The default implementation
    attempts to dispatch JSON-RPC calls to the functions or instance
    installed in the server. Override the _dispatch method inhereted
    from SimpleJSONRPCDispatcher to change this behavior.
    """
    allow_reuse_address = True

    def __init__(self, addr, requestHandler=SimpleJSONRPCRequestHandler,
                 logRequests=True):
        self.logRequests = logRequests
        if sys.version_info[:2] == (2, 4):
            SimpleJSONRPCDispatcher.__init__(self)
        else:
            SimpleJSONRPCDispatcher.__init__(self, allow_none=True, 
                                             encoding=None)
        SocketServer.TCPServer.__init__(self, addr, requestHandler)

        self.__serving = False  
        self.__thread = None  
        self.__is_shut_down = threading.Event()

    def serve_forever(self, in_thread=False):
        def serve_thread(server):
            server.serve_forever()
        if in_thread:
            self.__thread = threading.Thread(target=serve_thread, args=[self])
            self.__thread.setDaemon(True)
            self.__thread.start()
        else:
            self.__serving = True
            self.__is_shut_down.clear()
            while self.__serving:
                self.handle_request()
            self.__is_shut_down.set()

    def get_request(self):
        while True:
            r = select.select([self.socket], [], [], 1)
            if len(r[0]) > 0:
                return self.socket.accept()
            if not self.__serving:
                return (None, None)

    def verify_request(self, request, client_address):
        return request is not None

    def shutdown(self, immediately = True):
        self.__serving = False
        if not immediately:
            return
        self.__is_shut_down.wait()
        if self.__thread:
            self.__thread.join()
            self.__thread = None
